@@ -5,7 +5,7 @@ from fireo.fields.datetime_field import DateTime
 from fireo.fields.geo_point_field import GeoPoint
 from fireo.fields.list_field import ListField
 from fireo.fields.map_field import MapField
-from fireo.fields.nested_model_field import NestedModel
+from fireo.fields.nested_model_field import NestedModel, NestedModelField
 from fireo.fields.number_field import NumberField
 from fireo.fields.reference_field import ReferenceField
 from fireo.fields.text_field import TextField
