@@ -1,7 +1,7 @@
 from dataclasses import replace
 from typing import Any, List, Optional
 
-from google.cloud.firestore_v1 import ArrayRemove, ArrayUnion
+from google.cloud.firestore_v1.transforms import _ValueList
 
 from fireo.fields import errors, Field, IDField
 from fireo.utils.types import DumpOptions, LoadOptions
@@ -22,7 +22,7 @@ class ListField(Field):
     allowed_attributes = ['nested_field']
 
     def __init__(self, nested_field=None, *args, **kwargs):
-        kwargs['nested_field'] = nested_field
+        kwargs['nested_field'] = nested_field or Field()
         super().__init__(*args, **kwargs)
 
     def attr_nested_field(self, attr_val, field_val):
@@ -47,51 +47,52 @@ class ListField(Field):
 
         if val is None:
             return None
-
-        if type(val) not in [list, ArrayUnion, ArrayRemove]:
+        elif isinstance(val, list):
+            values = val
+        elif isinstance(val, _ValueList):
+            values = val.values
+        else:
             raise errors.InvalidFieldType(f'Invalid field type. Field "{self.name}" expected {list}, '
                                           f'got {type(val)}')
 
-        nested_field: Optional[Field] = self.raw_attributes.get('nested_field')
-        if nested_field is not None:
-            serialized_values = []
-            for index, item in enumerate(val):
-                try:
-                    serialized_values.append(nested_field.get_value(
-                        val=item,
-                        dump_options=replace(
-                            dump_options,
-                            # ignore_unchanged used in update. Object nested in list cannot be updated partially
-                            ignore_unchanged=False,
-                        )
-                    ))
-                except Exception as error:
-                    from fireo.models.errors import ModelSerializingWrappedError
-                    raise ModelSerializingWrappedError(item, (index,), error) from error
-        else:
-            serialized_values = val
+        nested_field: Field = self.raw_attributes.get('nested_field')
+        serialized_values = []
+        for index, item in enumerate(values):
+            try:
+                serialized_values.append(nested_field.get_value(
+                    val=item,
+                    dump_options=replace(
+                        dump_options,
+                        # ignore_unchanged used in update. Object nested in list cannot be updated partially
+                        ignore_unchanged=False,
+                    )
+                ))
+            except Exception as error:
+                from fireo.models.errors import ModelSerializingWrappedError
+                raise ModelSerializingWrappedError(item, (index,), error) from error
+
+        if isinstance(val, _ValueList):
+            serialized_values = type(val)(serialized_values)
 
         return self.db_value(serialized_values)
 
     def db_value(self, val):
-        # check if user defined to set the value as lower case
-        if self.model_cls._meta.to_lowercase:
-            return [v.lower() if type(v) is str else v for v in val]
-
         return val
 
     def field_value(self, val: Optional[List[Any]], load_options=LoadOptions()) -> Optional[List[Any]]:
         parsed = super().field_value(val, load_options)
-        nested_field: Optional[Field] = self.raw_attributes.get('nested_field')
+        nested_field: Field = self.raw_attributes.get('nested_field')
 
-        if parsed is not None and nested_field is not None:
-            parsed = [
-                nested_field.field_value(item, replace(
-                    load_options,
-                    merge=False,  # merge is not supported for list items
-                ))
-                for item in parsed
-            ]
+        if parsed is None:
+            return None
+
+        parsed = [
+            nested_field.field_value(item, replace(
+                load_options,
+                merge=False,  # merge is not supported for list items
+            ))
+            for item in parsed
+        ]
 
         return parsed
 
