@@ -1,17 +1,27 @@
-from datetime import datetime
+from enum import Enum
+from functools import partial
 
 from google.cloud import firestore
 from google.cloud.firestore_v1.field_path import FieldPath
 
 from fireo.database import db
-from fireo.fields import DateTime, ReferenceField
-from fireo.fields.errors import FieldNotFound
+from fireo.fields import MapField
 from fireo.queries import query_wrapper
 from fireo.queries.base_query import BaseQuery
 from fireo.queries.delete_query import DeleteQuery
 from fireo.queries.query_iterator import QueryIterator
 from fireo.utils import utils
-from fireo.utils.utils import get_db_column_names_for_path
+from fireo.utils.cursor import Cursor
+from fireo.utils.types import DumpOptions
+from fireo.utils.utils import (
+    get_dot_names_as_dot_columns,
+    get_nested_field_by_dotted_name,
+)
+
+
+class Ordering(str, Enum):
+    ASCENDING = "Asc"
+    DESCENDING = "Desc"
 
 
 class FilterQuery(BaseQuery):
@@ -27,10 +37,10 @@ class FilterQuery(BaseQuery):
     select_query: list
         Where filter
 
-    n_limit: int
+    limit: int
         Number of documents
 
-    oderer_by: list
+    order: list
         Order the documents
 
     query_transaction:
@@ -99,148 +109,131 @@ class FilterQuery(BaseQuery):
         Attach key to model for later updating the model
     """
 
-    def __init__(self, model_cls, parent=None, *args, **kwargs):
-        super().__init__(model_cls)
-        self.model = model_cls()
-        self.select_query = self._where_filter(*args, **kwargs)
-        self.n_limit = None
-        self._offset = None
-        self.order_by = []
+    def __init__(
+        self,
+        model_cls,
+        parent=None,
+        *,
+        # private use
+        select_query=None,
+        limit=None,
+        offset=None,
+        order=None,
+        start_after=None,
+        start_at=None,
+        end_before=None,
+        end_at=None,
+        query_transaction=None,
+        query_batch=None,
+        **kwargs,
+    ):
+        super().__init__(model_cls, **kwargs)
         self.parent = parent
-        self.cursor_dict = {}
-        self._start_after = None
-        self._start_at = None
-        self._end_before = None
-        self._end_at = None
-        self.query_transaction = None
-        self.query_batch = None
+        self._select_query = (select_query or []).copy()
+        self._limit = limit
+        self._offset = offset
+        self._order = (order or []).copy()
+        self._start_after = start_after
+        self._start_at = start_at
+        self._end_before = end_before
+        self._end_at = end_at
+        self._query_transaction = query_transaction
+        self._query_batch = query_batch
         if parent:
-            super().set_collection_path(path=parent)
-            # Add parent in cursor
-            self.cursor_dict['parent'] = parent
+            self.set_collection_path(path=parent)
 
-    def _where_filter(self, *args, **kwargs):
-        """For Direct assign of filter when equality operator"""
-        if args:
-            return [args]
-        elif kwargs:
-            filter = []
-            for k, v in kwargs.items():
-                filter.append((k, '==', v))
-            return filter
-        else:
-            return []
+    def _deconstruct(self):
+        return {
+            **super()._deconstruct(),
+            'parent': self.parent,
+            'select_query': self._select_query,
+            'limit': self._limit,
+            'offset': self._offset,
+            'order': self._order,
+            'start_after': self._start_after,
+            'start_at': self._start_at,
+            'end_before': self._end_before,
+            'end_at': self._end_at,
+            'query_transaction': self._query_transaction,
+            'query_batch': self._query_batch,
+        }
 
     def transaction(self, t):
-        self.query_transaction = t
-        return self
+        return self.copy(query_transaction=t)
 
     def batch(self, b):
-        self.query_batch = b
-        return self
+        return self.copy(query_batch=b)
 
-    def parse_where(self):
+    def _parse_where(self):
         """Parse where filter
 
         User can change the field column name in firestore So, filter where clauses
         on the base of db column name
         """
         filters = []
-        for w in self.select_query:
+        for w in self._select_query:
             name, op, val = w
 
-            # save the filter in cursor for next fetch
-            #
-            # ISSUE # 77
-            # if filter value type is datetime then it need to first
-            # convert into string then JSON serialize
-            cf = w
-            if type(val) is datetime:
-                cf = (name, op, val.isoformat())
-
-            if 'filters' in self.cursor_dict:
-                self.cursor_dict['filters'].append(cf)
-            else:
-                self.cursor_dict['filters'] = [cf]
-
-            try:
-                # ISSUE # 77
-                # if field is datetime and type is str (which is usually come from cursor)
-                # then convert this string into datetime format
-                if isinstance(self.model._meta.get_field(name), DateTime) and type(val) is str:
-                    val = datetime.fromisoformat(val)
-
-                # ISSUE # 78
-                # check if field is ReferenceField then to query this field we have to
-                # convert this value into document reference then filter it
-                if isinstance(self.model._meta.get_field(name), ReferenceField):
-                    # ISSUE # 116
-                    # ReferenceFields can be optional and None
-                    # in which case, do not update val to a doc that doesn't exist
-                    if val is not None:
-                        val = db.conn.document(val)
-            except FieldNotFound:
-                # Filter with nested model not able to find the field (e.g user.name)
-                # it require to loop the nested model first to find the field
-                # so just ignore it
-                pass
-
-            # check if user defined to set the value as lower case
-            if self.model._meta.to_lowercase and type(val) is str:
-                val = val.lower()
-
             # ISSUE # 160
             # check if it is ID field
             if self._is_id_field(name):
                 # should yield "__name__"
-                f_name = FieldPath.document_id()
+                db_col_name = FieldPath.document_id()
 
-                # value should be an array
+                # value should an reference
                 if type(val) is list:
                     val = [self._get_ref_by_key_or_id(v) for v in val]
                 else:
                     val = self._get_ref_by_key_or_id(val)
             else:
-                f_name = self._get_db_column_name(name)
-            filters.append((f_name, op, val))
+                field = get_nested_field_by_dotted_name(self.model_cls, name)
+                db_col_name = get_dot_names_as_dot_columns(self.model_cls, name)
+
+                if not isinstance(field, MapField):
+                    serialise_value = partial(
+                        field.get_value,
+                        dump_options=DumpOptions(ignore_required=True, ignore_default=True)
+                    )
+                    if op == 'array_contains':
+                        val = serialise_value([val])[0]
+                    elif op in ('in', 'not_in'):
+                        val = [serialise_value(v) for v in val]
+                    else:
+                        val = serialise_value(val)
+
+            filters.append((db_col_name, op, val))
         return filters
 
     def _get_ref_by_key_or_id(self, key_or_id):
         """Get document reference by key or id"""
         key = key_or_id
         if not utils.is_key(key_or_id):
-            key = self.model.collection_name + "/" + key_or_id
-            if self.model.parent:
-                key = self.model.parent + "/" + key
+            key = utils.get_key(
+                collection=self.model_cls.collection_name,
+                doc_id=key_or_id,
+                parent_key=self.model_cls.parent,
+            )
 
         return db.conn.document(key)
 
-    def _get_db_column_name(self, name):
-        model = self.model
-        db_field_path = get_db_column_names_for_path(model, *name.split('.'))
-
-        f_name = '.'.join(db_field_path)
-        return f_name
-
+    @property
     def query(self):
         """Make a query for firestore"""
         ref = self.get_ref()
         # parse where filter
-        for f in self.parse_where():
+        for f in self._parse_where():
             ref = ref.where(*f)
         # Apply limit
-        if self.n_limit:
-            ref = ref.limit(self.n_limit)
+        if self._limit:
+            ref = ref.limit(self._limit)
         # Apply offset
         if self._offset:
             ref = ref.offset(self._offset)
         # order the documents
-        for o in self.order_by:
+        for o in self._order:
             name, direction = o
-            if direction == 'Desc':
-                ref = ref.order_by(name, direction=firestore.Query.DESCENDING)
-            else:
-                ref = ref.order_by(name)
+            db_column = get_dot_names_as_dot_columns(self.model_cls, name)
+            ref = ref.order_by(db_column, direction=direction)
 
         if self._start_after:
             ref = ref.start_after(self._start_after)
@@ -256,7 +249,7 @@ class FilterQuery(BaseQuery):
     def _fields_by_column_name(self, **kwargs):
         """Change the field name according to their db column name"""
         return {
-            self.model._meta.get_field(k).db_column_name: v
+            self.model_cls._meta.get_field(k).db_column_name: v
             for k, v in kwargs.items()
         }
 
@@ -272,7 +265,7 @@ class FilterQuery(BaseQuery):
 
         # Checking `model._meta.id` because `model._id` or `model.id` are not
         # populated on CLS
-        field_name, field = self.model._meta.id
+        field_name, field = self.model_cls._meta.id
         if name == field_name and not field.include_in_document:
             return True
 
@@ -284,35 +277,53 @@ class FilterQuery(BaseQuery):
 
     def start_after(self, key=None, **kwargs):
         """Start document after this"""
+        self._assert_position_key_or_kwargs(key, kwargs)
+
         if key:
-            self._start_after = self._firestore_doc(key)
+            start_after = self._firestore_doc(key)
         else:
-            self._start_after = self._fields_by_column_name(**kwargs)
-        return self
+            start_after = self._fields_by_column_name(**kwargs)
+
+        return self.copy(start_after=start_after)
 
     def start_at(self, key=None, **kwargs):
         """Start document at this point"""
+        self._assert_position_key_or_kwargs(key, kwargs)
+
         if key:
-            self._start_at = self._firestore_doc(key)
+            start_at = self._firestore_doc(key)
         else:
-            self._start_at = self._fields_by_column_name(**kwargs)
-        return self
+            start_at = self._fields_by_column_name(**kwargs)
+
+        return self.copy(start_at=start_at)
 
     def end_before(self, key=None, **kwargs):
         """End document before this point"""
+        self._assert_position_key_or_kwargs(key, kwargs)
+
         if key:
-            self._end_before = self._firestore_doc(key)
+            end_before = self._firestore_doc(key)
         else:
-            self._end_before = self._fields_by_column_name(**kwargs)
-        return self
+            end_before = self._fields_by_column_name(**kwargs)
+
+        return self.copy(end_before=end_before)
 
     def end_at(self, key=None, **kwargs):
         """End document at this point"""
+        self._assert_position_key_or_kwargs(key, kwargs)
+
         if key:
-            self._end_at = self._firestore_doc(key)
+            end_at = self._firestore_doc(key)
         else:
-            self._end_at = self._fields_by_column_name(**kwargs)
-        return self
+            end_at = self._fields_by_column_name(**kwargs)
+
+        return self.copy(end_at=end_at)
+
+    def _assert_position_key_or_kwargs(self, key, kwargs):
+        """Assert if key or kwargs is provided"""
+        assert key or kwargs, 'You must provide either a key or kwargs'
+        assert not (key and kwargs), 'You must provide either a key or kwargs'
+        assert not key or utils.is_key(key), 'Key must be a valid key'
 
     def filter(self, *args, **kwargs):
         """Apply filter for querying document
@@ -332,26 +343,23 @@ class FilterQuery(BaseQuery):
         self:
             Return self object
         """
+        select_query = []
         if args:
-            self.select_query.append(args)
+            select_query.append(args)
+
         elif kwargs:
             for k, v in kwargs.items():
-                self.select_query.append((k, '==', v))
-        return self
+                select_query.append((k, '==', v))
+
+        return self.copy(select_query=[*self._select_query, *select_query])
 
     def limit(self, count):
         """Apply limit for query"""
-        # save the Limit in cursor for next fetch
-        self.cursor_dict['limit'] = count
-
-        if count:
-            self.n_limit = count
-        return self
+        return self.copy(limit=count)
 
     def offset(self, num_to_skip):
         """Offset for query"""
-        self._offset = num_to_skip
-        return self
+        return self.copy(offset=num_to_skip)
 
     def order(self, field_name):
         """Order document by field name
@@ -372,24 +380,15 @@ class FilterQuery(BaseQuery):
         -------
             Self object
         """
-        # Save order in cursor dict for next fetch
-        if 'order' in self.cursor_dict:
-            self.cursor_dict['order'] = self.cursor_dict['order'] + ',' + field_name
-        else:
-            self.cursor_dict['order'] = field_name
-
-        order_direction = 'Asc'
+        order_direction = firestore.Query.ASCENDING
         name = field_name
 
         # If this is in Desc order
         if field_name[0] == '-':
-            order_direction = 'Desc'
+            order_direction = firestore.Query.DESCENDING
             name = field_name[1:]  # Get the field name after dash(-) e.g -age name will be age
 
-        f_name = self._get_db_column_name(name)
-
-        self.order_by.append((f_name, order_direction))
-        return self
+        return self.copy(order=[*self._order, (name, order_direction)])
 
     def fetch(self, limit=None):
         """Fetch the result from firestore
@@ -399,16 +398,19 @@ class FilterQuery(BaseQuery):
         limit : optional
             Apply limit to firestore documents, how much documents you want to retrieve
         """
-        # save the Limit in cursor for next fetch
-        self.cursor_dict['limit'] = limit
-
+        fq = self
         if limit:
-            self.n_limit = limit
-        return QueryIterator(self)
+            fq = self.copy(limit=limit)
+        return QueryIterator(
+            model_cls=fq.model_cls,
+            query=fq.query,
+            query_transaction=fq._query_transaction,
+            limit=fq._limit,
+            cursor=Cursor.extract(fq),
+        )
 
     def group_fetch(self, limit=None):
-        super().set_group_collection(True)
-        return self.fetch(limit)
+        return self.copy(group_collection=True).fetch(limit)
 
     def get(self):
         """Get the first matching document from firestore
@@ -417,12 +419,12 @@ class FilterQuery(BaseQuery):
         This is same as `fetch(limit=1)` the only difference is `get()` method
         return **model instance** and the `fetch()` method return the **generator**
         """
-        self.n_limit = 1
-        doc = next(self.query().stream(self.query_transaction), None)
+        filter_query = self.copy(limit=1)
+        doc = next(filter_query.query.stream(filter_query._query_transaction), None)
         if doc:
-            m = query_wrapper.ModelWrapper.from_query_result(self.model, doc)
-            m._update_doc = self._update_doc_key(m)
+            m = query_wrapper.ModelWrapper.from_query_result(filter_query.model_cls(), doc)
             return m
+
         return None
 
     def delete(self, child=False):
@@ -430,9 +432,8 @@ class FilterQuery(BaseQuery):
 
         if child is True then delete nested collection and documents also
         """
-        transaction_or_batch = self.query_transaction if self.query_transaction else self.query_batch
-        q = self.query()
-        DeleteQuery(self.model, query=q, child=child).exec(transaction_or_batch)
+        transaction_or_batch = self._query_transaction if self._query_transaction else self._query_batch
+        DeleteQuery(self.model_cls, query=self.query, child=child).exec(transaction_or_batch)
 
     def _update_doc_key(self, model):
         """Attach key to model for later updating the model
