@@ -1,7 +1,13 @@
-import base64
-import json
 import itertools
+from typing import Optional, TYPE_CHECKING
+
+from google.cloud.firestore_v1 import CollectionReference, Transaction
+
 from fireo.queries import query_wrapper
+from fireo.utils.cursor import Cursor
+
+if TYPE_CHECKING:
+    pass
 
 
 class QueryIterator:
@@ -16,13 +22,24 @@ class QueryIterator:
         Fetch next results
     """
 
-    def __init__(self, query):
+    def __init__(
+        self,
+        model_cls,
+        query: CollectionReference,
+        query_transaction: Optional[Transaction],
+        limit: Optional[int],
+        cursor: Cursor,
+    ):
         self.query = query
-        self.model_cls = query.model.__class__
-        self.docs = query.query().stream(query.query_transaction)
+        self.model_cls = model_cls
+        self.docs = query.stream(query_transaction)
+        self.limit = limit
+
         # Get offset for next fetch
-        self.offset = query.n_limit
-        query.cursor_dict['offset'] = query.n_limit
+        self.offset = limit
+        self._cursor = cursor
+        self._cursor['offset'] = self.offset
+
         # Hold the last doc for next fetch
         self.last_doc = None
         self.last_doc_key = None
@@ -38,13 +55,13 @@ class QueryIterator:
                 # Suppose this is the last doc
                 self.last_doc = doc
                 m = query_wrapper.ModelWrapper.from_query_result(self.model_cls(), doc)
-                m._update_doc = self.query._update_doc_key(m)
                 # Suppose this is last doc
                 self.last_doc_key = m.key
                 return m
             self.fetch_end = True
             # Save last doc key in cursor
-            self.query.cursor_dict['last_doc_key'] = self.last_doc_key
+            if self.last_doc_key:
+                self._cursor['last_doc_key'] = self.last_doc_key
             raise StopIteration
         except StopIteration:
             raise StopIteration
@@ -57,23 +74,22 @@ class QueryIterator:
             # check if fetch end then use last doc otherwise use the offset
             if self.fetch_end:
                 self.fetch_end = False
-                q = self.query.query().start_after(self.last_doc)
+                q = self.query.start_after(self.last_doc)
             else:
-                q = self.query.query().offset(self.offset)
+                q = self.query.offset(self.offset)
 
             # Apply new Limit if there is any
             if limit:
                 q = q.limit(limit)
                 self.offset += limit
             else:
-                self.offset += self.query.n_limit
+                self.offset += self.limit
 
             # Update offset in cursor
-            self.query.cursor_dict['offset'] = self.offset
+            self._cursor['offset'] = self.offset
 
             self.docs = itertools.chain(self.docs, q.stream())
 
     @property
     def cursor(self):
-        encodedCursor = base64.b64encode(json.dumps(self.query.cursor_dict).encode('utf-8'))
-        return str(encodedCursor, 'utf-8')
+        return self._cursor.to_string()
