@@ -1,9 +1,14 @@
+from typing import Optional, Type, TYPE_CHECKING
+
 from fireo.queries import query_wrapper
 from fireo.queries.base_query import BaseQuery
 from fireo.utils import utils
 from fireo.utils.types import DumpOptions
 from fireo.utils.utils import get_flat_dict
 
+if TYPE_CHECKING:
+    from fireo.models import Model
+
 
 class UpdateQuery(BaseQuery):
     """Update document in firestore
@@ -23,10 +28,34 @@ class UpdateQuery(BaseQuery):
         return modified instance of model
     """
 
-    def __init__(self, model_cls, mutable_instance):
+    def __init__(
+        self,
+        model_cls: 'Type[Model]',
+        mutable_instance: 'Optional[Model]' = None,
+        no_return: bool = False,
+        key: Optional[str] = None,
+        values: Optional[dict] = None,
+    ):
         super().__init__(model_cls)
+        assert mutable_instance is None or isinstance(mutable_instance, model_cls), (
+            'mutable_instance must be instance of model_cls'
+        )
+        assert mutable_instance is not None or key is not None, (
+            'mutable_instance or key is required'
+        )
+
         self.model = mutable_instance
-        super().set_collection_path(key=mutable_instance.key)
+        if self.model is None:
+            self.model = model_cls()
+
+        if key is not None:
+            self.model.key = key
+
+        if values is not None:
+            self.model.merge_with_dict(values)
+
+        self.no_return = no_return
+        super().set_collection_path(key=self.model.key)
 
     def _doc_ref(self):
         """create document ref from firestore"""
@@ -62,15 +91,23 @@ class UpdateQuery(BaseQuery):
     def _raw_exec(self, transaction_or_batch=None):
         """Update document in firestore and return the document"""
         ref = self._doc_ref()
-        self.model._id = ref.id  #
+        self.model._id = ref.id
+        values = self._parse_field()
+
         if transaction_or_batch is not None:
-            transaction_or_batch.update(ref, self._parse_field())
+            if values:
+                transaction_or_batch.update(ref, values)
             return ref
 
-        parse_field = self._parse_field()
-        if parse_field:
-            ref.update(parse_field)
-            return ref.get()
+        if values:
+            ref.update(values)
+
+        self.model._reset_field_changed()
+
+        if self.no_return:
+            return None
+
+        return ref.get()
 
     def exec(self, transaction_or_batch=None):
         """return modified instance of model"""
